/*
 * Copyright (c) 2005-2012, Claybird
 * All rights reserved.

 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:

 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the Claybird nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#include "stdafx.h"
#include "Archiver7ZIP.h"
#include "../Utilities/FileOperation.h"
#include "../Utilities/OSUtil.h"
#include "../Utilities/StringUtil.h"
#include "../Utilities/TemporaryDirMgr.h"
#include "../resource.h"
#include "../ConfigCode/ConfigExtract.h"
#include "../ConfigCode/Config7Z.h"
#include "../ConfigCode/ConfigZIP.h"
#include "../Dialogs/SevenZipVolumeSizeDlg.h"
#include "../Utilities/DummyWindow.h"

CArchiver7ZIP::CArchiver7ZIP():
	ArchiverSetUnicodeMode(NULL),
	ArchiverGetArchiveType(NULL)
{
	m_nRequiredVersion=920;
	m_nRequiredSubVersion=2;
	m_strDllName=_T("7-ZIP32.DLL");
	m_AstrPrefix="SevenZip";
}

CArchiver7ZIP::~CArchiver7ZIP()
{
	FreeDLL();
}

LOAD_RESULT CArchiver7ZIP::LoadDLL(CConfigManager &ConfMan,CString &strErr)
{
	FreeDLL();

	//NX̃\bhĂ
	LOAD_RESULT res=CArchiverDLL::LoadDLL(ConfMan,strErr);
	if(LOAD_RESULT_OK!=res){
		return res;
	}

	//UNICODE[hݒp
	CStringA strFunctionName;
	strFunctionName=m_AstrPrefix+"SetUnicodeMode";
	ArchiverSetUnicodeMode=(COMMON_ARCHIVER_SETUNICODEMODE)GetProcAddress(m_hInstDLL,strFunctionName);
	if(NULL==ArchiverSetUnicodeMode){
		strErr.Format(IDS_ERROR_DLL_FUNCTION_GET,m_strDllName,(LPCTSTR)CA2T(strFunctionName));
		FreeDLL();
		TRACE(_T("%s\n"),(LPCTSTR)strErr);
		return LOAD_RESULT_INVALID;
	}
	//UNICODE[hɐݒ
	if(!ArchiverSetUnicodeMode(TRUE)){
		//UNICODEɂłȂ玸sƂ
		return LOAD_RESULT_INVALID;
	}

	//A[JCuʔp
	strFunctionName=m_AstrPrefix+"GetArchiveType";
	ArchiverGetArchiveType=(COMMON_ARCHIVER_GETARCHIVETYPE)GetProcAddress(m_hInstDLL,strFunctionName);
	if(NULL==ArchiverGetArchiveType){
		strErr.Format(IDS_ERROR_DLL_FUNCTION_GET,m_strDllName,(LPCTSTR)CA2T(strFunctionName));
		FreeDLL();
		TRACE(_T("%s\n"),(LPCTSTR)strErr);
		return LOAD_RESULT_INVALID;
	}
	return LOAD_RESULT_OK;
}

void CArchiver7ZIP::FreeDLL()
{
	if(m_hInstDLL){
		ArchiverSetUnicodeMode=NULL;
		ArchiverGetArchiveType=NULL;
		CArchiverDLL::FreeDLL();
	}
}

bool CArchiver7ZIP::FormatCompressCommandZIP(const CConfigZIP &ConfZIP,CString &Param,bool bZIPSFX,int Options,LPCTSTR lpszMethod,LPCTSTR lpszLevel,CString &strLog)
{
	Param+=
		_T("a ")			//k
		_T("-tzip ")		//ZIP`ňk
		_T("-r0 ")			//TufBNg
	;
	bool bBadSFX=false;
	//k
	if(lpszMethod && *lpszMethod!=_T('\0')){
		Param+=_T("-mm=");
		Param+=lpszMethod;
		Param+=_T(" ");
	}else{
		switch(ConfZIP.CompressType){
		case ZIP_COMPRESS_DEFLATE:
			Param+=_T("-mm=Deflate ");
			break;
		case ZIP_COMPRESS_DEFLATE64:
			Param+=_T("-mm=Deflate64 ");
			break;
		case ZIP_COMPRESS_BZIP2:
			bBadSFX=true;
			Param+=_T("-mm=BZip2 ");
			break;
		case ZIP_COMPRESS_COPY:
			Param+=_T("-mm=Copy ");
			break;
		case ZIP_COMPRESS_LZMA:
			bBadSFX=true;
			Param+=_T("-mm=LZMA ");
			break;
		case ZIP_COMPRESS_PPMD:
			bBadSFX=true;
			Param+=_T("-mm=PPMd ");
			break;
		}
	}
	//kx
	if(lpszLevel && *lpszLevel!=_T('\0')){
		Param+=_T("-mx=");
		Param+=lpszLevel;
		Param+=_T(" ");
	}else{
		switch(ConfZIP.CompressLevel){
		case ZIP_COMPRESS_LEVEL0:
			Param+=_T("-mx=0 ");
			break;
		case ZIP_COMPRESS_LEVEL5:
			Param+=_T("-mx=5 ");
			break;
		case ZIP_COMPRESS_LEVEL9:
			Param+=_T("-mx=9 ");
			break;
		}
	}
	if((ZIP_COMPRESS_DEFLATE==ConfZIP.CompressType)||(ZIP_COMPRESS_DEFLATE64==ConfZIP.CompressType)){
		if(ConfZIP.SpecifyDeflateMemorySize){
			CString temp;
			temp.Format(_T("-mfb=%d "),ConfZIP.DeflateMemorySize);
			Param+=temp;
		}
		if(ConfZIP.SpecifyDeflatePassNumber){
			CString temp;
			temp.Format(_T("-mpass=%d "),ConfZIP.DeflatePassNumber);
			Param+=temp;
		}
	}
	//R[h
	if(ConfZIP.ForceUTF8){
		bBadSFX=true;
		Param+=_T("-mcu=on ");
	}
	if(Options&COMPRESS_PASSWORD){	//pX[ht
		Param+=_T("-p ");

		//Í[h
		switch(ConfZIP.CryptoMode){
		case ZIP_CRYPTO_ZIPCRYPTO:
			Param+=_T("-mem=ZipCrypto ");
			break;
		case ZIP_CRYPTO_AES128:
			bBadSFX=true;
			Param+=_T("-mem=AES128 ");
			break;
		case ZIP_CRYPTO_AES192:
			bBadSFX=true;
			Param+=_T("-mem=AES192 ");
			break;
		case ZIP_CRYPTO_AES256:
			bBadSFX=true;
			Param+=_T("-mem=AES256 ");
			break;
		}
	}

	if(bZIPSFX){
		//BZip2̎ȉ𓀂̓T|[gĂȂ
		if(bBadSFX){
			strLog=CString(MAKEINTRESOURCE(IDS_ERROR_ZIP_SFX_DONT_SUPPORT_FORMAT));
			return false;
		}
	}

	return true;
}

bool CArchiver7ZIP::FormatCompressCommand7Z(const CConfig7Z &Conf7Z,CString &Param,int Options,LPCTSTR lpszMethod,CString &strLog)
{
	Param+=
		_T("a ")			//k
		_T("-t7z ")			//7-Zip`ňk
		_T("-r0 ")			//TufBNg
	;
	if(lpszMethod && *lpszMethod!=_T('\0')){
		Param+=_T("-m0=");
		Param+=lpszMethod;
		Param+=_T(" ");
	}else{
		if(Conf7Z.UsePreset){
			//vZbgk[h
			switch(Conf7Z.CompressLevel){	//kx
			case SEVEN_ZIP_COMPRESS_LEVEL0:
				Param+=_T("-mx=0 ");
				break;
			case SEVEN_ZIP_COMPRESS_LEVEL1:
				Param+=_T("-mx=1 ");
				break;
			case SEVEN_ZIP_COMPRESS_LEVEL5:
				Param+=_T("-mx=5 ");
				break;
			case SEVEN_ZIP_COMPRESS_LEVEL7:
				Param+=_T("-mx=7 ");
				break;
			case SEVEN_ZIP_COMPRESS_LEVEL9:
				Param+=_T("-mx=9 ");
				break;
			}
		}else{
			switch(Conf7Z.CompressType){	//k
			case SEVEN_ZIP_COMPRESS_LZMA:
				Param+=_T("-m0=LZMA:a=");
				switch(Conf7Z.LZMA_Mode){	//LZMAk[h
				case SEVEN_ZIP_LZMA_MODE0:
					Param+=_T("0 ");
					break;
				case SEVEN_ZIP_LZMA_MODE1:
					Param+=_T("1 ");
					break;
				//case SEVEN_ZIP_LZMA_MODE2:
				//	Param+=_T("2 ");
				//	break;
				}
				break;
			case SEVEN_ZIP_COMPRESS_PPMD:
				Param+=_T("-m0=PPMd ");
				break;
			case SEVEN_ZIP_COMPRESS_BZIP2:
				Param+=_T("-m0=BZip2 ");
				break;
			case SEVEN_ZIP_COMPRESS_DEFLATE:
				Param+=_T("-m0=Deflate ");
				break;
			case SEVEN_ZIP_COMPRESS_COPY:
				Param+=_T("-m0=Copy ");
				break;
			case SEVEN_ZIP_COMPRESS_LZMA2:
				Param+=_T("-m0=LZMA2:a=");
				switch(Conf7Z.LZMA_Mode){	//LZMAk[h
				case SEVEN_ZIP_LZMA_MODE0:
					Param+=_T("0 ");
					break;
				case SEVEN_ZIP_LZMA_MODE1:
					Param+=_T("1 ");
					break;
				}
			}
		}
	}
	if(Conf7Z.SolidMode){	//\bh[h
		Param+=_T("-ms=on ");
	}else{
		Param+=_T("-ms=off ");
	}
	//------------
	// wb_k
	//------------
	if(Conf7Z.HeaderCompression){	//wb_k
		Param+=_T("-mhc=on ");
	}else{
		Param+=_T("-mhc=off ");
		Param+=_T("-mhcf=off ");
	}

	if(Options&COMPRESS_PASSWORD){	//pX[ht
		Param+=_T("-p ");
		if(Conf7Z.HeaderEncryption){	//wb_Í
			Param+=_T("-mhe=on ");
		}
		else{
			Param+=_T("-mhe=off ");
		}
	}
	if(Options&COMPRESS_SFX){
		//BZip2̎ȉ𓀂̓T|[gĂȂ
		if(SEVEN_ZIP_COMPRESS_BZIP2==Conf7Z.CompressType){
			strLog=CString(MAKEINTRESOURCE(IDS_ERROR_7Z_SFX_DONT_SUPPORT_BZIP2));
			return false;
		}
		Param+=_T("-sfx ");
	}
	return true;
}

bool CArchiver7ZIP::Compress(LPCTSTR ArcFileName,std::list<CString> &ParamList,CConfigManager &ConfMan,const PARAMETER_TYPE Type,int Options,LPCTSTR lpszFormat,LPCTSTR lpszMethod,LPCTSTR lpszLevel,CString &strLog)
{
	if(!IsOK()){
		return false;
	}

	ASSERT(0!=_tcslen(ArcFileName));
	TRACE(_T("ArcFileName=%s\n"),ArcFileName);

	//ɂ̏ꍇ͎ȉ𓀂͏oȂ
	if((Options&COMPRESS_SPLIT)&&(Options&COMPRESS_SFX)){
		ErrorMessage(CString(MAKEINTRESOURCE(IDS_ERROR_CANNOT_SPLIT_SFX)));
		strLog=CString(MAKEINTRESOURCE(IDS_ERROR_CANNOT_SPLIT_SFX));
		Options&=~COMPRESS_SFX;
	}

	//==============================================
	// X|Xt@Cpe|t@C擾
	//==============================================
	TCHAR ResponceFileName[_MAX_PATH+1];
	FILL_ZERO(ResponceFileName);
	if(!UtilGetTemporaryFileName(ResponceFileName,_T("zip"))){
		strLog=CString(MAKEINTRESOURCE(IDS_ERROR_TEMPORARY_FILE_CREATE));
		return false;
	}
	ASSERT(0!=_tcslen(ResponceFileName));

	//============================================
	// ȉ𓀃t@Cpe|t@C擾
	//============================================
	TCHAR SFXTemporaryFileName[_MAX_PATH+1];
	FILL_ZERO(SFXTemporaryFileName);
	TCHAR SFXModulePath[_MAX_PATH+1];
	FILL_ZERO(SFXModulePath);
	bool bZIPSFX=((0!=(Options&COMPRESS_SFX))&&(PARAMETER_ZIP==Type));
	if(bZIPSFX){
		//2iK쐬
		if(!UtilGetTemporaryFileName(SFXTemporaryFileName,_T("sfx"))){
			strLog=CString(MAKEINTRESOURCE(IDS_ERROR_TEMPORARY_FILE_CREATE));
			return false;
		}
		ASSERT(0!=_tcslen(SFXTemporaryFileName));
		DeleteFile(SFXTemporaryFileName);//S~t@C

		//---SFXW[
		LPTSTR lptemp;
		{
			//SȃpXɈړ;DLLǂݍݑ΍
			CCurrentDirManager cdm(UtilGetModuleDirectoryPath());
			if(!SearchPath(NULL,_T("SFX32GUI.DAT"),NULL,_MAX_PATH,SFXModulePath,&lptemp)){
				strLog.Format(IDS_ERROR_SFX_MODULE_NOT_FOUND,_T("SFX32GUI.DAT"));
				return false;
			}
		}
	}

	//====================================================
	// X|Xt@CɈkΏۃt@CL
	// A[JCut@C̓R}hCŒڎw肷
	//====================================================
	{
		HANDLE hFile=CreateFile(ResponceFileName,GENERIC_WRITE,0,NULL,CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL,NULL);
		if(INVALID_HANDLE_VALUE==hFile){
			strLog=CString(MAKEINTRESOURCE(IDS_ERROR_TEMPORARY_FILE_ACCESS));
			return false;
		}

		TRACE(_T("X|Xt@Cւ̏\n"));
		std::list<CString>::iterator ite;
		for(ite=ParamList.begin();ite!=ParamList.end();ite++){
			CPath strPath=*ite;

			if(strPath.IsDirectory()){
				strPath.Append(_T("*"));
			}
			WriteResponceFile(hFile,strPath);
		}
		CloseHandle(hFile);
	}

	//===========================
	// DLLɓnIvV̐ݒ
	//===========================
	TRACE(_T("DLLɓnIvV̐ݒ\n"));

	CString Param;//R}hC p[^ obt@

	CConfigZIP confZIP;
	CConfig7Z  conf7Z;
	switch(Type){
	case PARAMETER_ZIP:	//ZIP`ňk
		confZIP.load(ConfMan);
		break;
	case PARAMETER_7Z:	//7z`ňk
		conf7Z.load(ConfMan);
		break;
	}

	switch(Type){
	case PARAMETER_ZIP:	//ZIP`ňk
		if(!FormatCompressCommandZIP(confZIP,Param,bZIPSFX,Options,lpszMethod,lpszLevel,strLog)){
			DeleteFile(ResponceFileName);
			return false;
		}
		break;
	case PARAMETER_7Z:	//7z`ňk
		if(!FormatCompressCommand7Z(conf7Z,Param,Options,lpszMethod,strLog)){
			DeleteFile(ResponceFileName);
			return false;
		}
		break;
	}
	//
	if(Options&COMPRESS_SPLIT){
		C7Zip32VolumeSizeDialog vsd;
		if(IDOK!=vsd.DoModal())return false;
		CString temp;
		temp.Format(_T("-v%d%s "),vsd.VolumeSize,ZIP_VOLUME_UNIT[vsd.SelectIndex].ParamName);
		Param+=temp;
	}

	Param+=_T("-scsUTF-8 ");	//X|Xt@C̃R[hy[Ww

	//kt@Cw
	if(bZIPSFX){
		Param+=_T("\"");
		Param+=SFXTemporaryFileName;
		Param+=_T("\" ");
	}
	else{
		Param+=_T("\"");
		Param+=ArcFileName;
		Param+=_T("\" ");
	}

	//X|Xt@Cw
	Param+=_T("\"@");
	Param+=ResponceFileName;
	Param+=_T("\"");

	ASSERT(!Param.IsEmpty());
	TRACE(_T("ArchiveHandler Commandline Parameter:%s\n"),Param);

	TRACE(_T("ArchiveHandlerĂяo\n"));
	std::vector<BYTE> szLog(LOG_BUFFER_SIZE);
	szLog[0]='\0';
	int Ret=ArchiveHandler(NULL,C2UTF8(Param),(LPSTR)&szLog[0],LOG_BUFFER_SIZE-1);
	CString strTmp;
	UtilToUNICODE(strTmp,&szLog[0],szLog.size()-1,UTILCP_UTF8);
	//strLog=&szLog[0];
	strLog=strTmp;

	//gX|Xt@C͏
	DeleteFile(ResponceFileName);

	//G[Oo
	if(!bZIPSFX||0!=Ret){
		if(bZIPSFX){
			DeleteFile(SFXTemporaryFileName);
		}
		return 0==Ret;
	}

	//==================================
	// ȉ𓀏ɂɕϊ(oCi)
	//==================================
	//o͐t@CJ
	HANDLE hArcFile=CreateFile(ArcFileName,GENERIC_WRITE,0,NULL,CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL,NULL);
	if(INVALID_HANDLE_VALUE==hArcFile){
		strLog.Format(IDS_ERROR_ACCESS_OUTPUT_FILE,ArcFileName);
		DeleteFile(ArcFileName);
		DeleteFile(SFXTemporaryFileName);
		return false;
	}
	{
		//SFXW[ǂݎ胂[hŊJ
		HANDLE hSFXModuleFile=CreateFile(SFXModulePath,GENERIC_READ,FILE_SHARE_READ,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
		if(INVALID_HANDLE_VALUE==hSFXModuleFile){
			strLog.Format(IDS_ERROR_SFX_MODULE_CANNOT_ACCESS,SFXModulePath);
			CloseHandle(hArcFile);
			DeleteFile(ArcFileName);
			DeleteFile(SFXTemporaryFileName);
			return false;
		}
		//SFXW[̒gRs[
		int Result=UtilAppendFile(hArcFile,hSFXModuleFile);
		if(Result>0){	//ǂݎG[
			strLog.Format(IDS_ERROR_SFX_MODULE_CANNOT_ACCESS,SFXModulePath);
			CloseHandle(hArcFile);
			CloseHandle(hSFXModuleFile);
			DeleteFile(ArcFileName);
			DeleteFile(SFXTemporaryFileName);
			return false;
		}
		else if(Result<0){	//݃G[
			strLog.Format(IDS_ERROR_ACCESS_OUTPUT_FILE,ArcFileName);
			CloseHandle(hArcFile);
			CloseHandle(hSFXModuleFile);
			DeleteFile(ArcFileName);
			DeleteFile(SFXTemporaryFileName);
			return false;
		}
		CloseHandle(hSFXModuleFile);
	}
	{
		//e|t@Cǂݎ胂[hŊJ
		HANDLE hSFXTempFile=CreateFile(SFXTemporaryFileName,GENERIC_READ,0,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
		if(INVALID_HANDLE_VALUE==hSFXTempFile){
			strLog=CString(MAKEINTRESOURCE(IDS_ERROR_TEMPORARY_FILE_ACCESS));
			CloseHandle(hArcFile);
			DeleteFile(ArcFileName);
			DeleteFile(SFXTemporaryFileName);
			return false;
		}
		//e|t@C̒gRs[
		int Result=UtilAppendFile(hArcFile,hSFXTempFile);
		if(Result>0){	//ǂݎG[
			strLog=CString(MAKEINTRESOURCE(IDS_ERROR_TEMPORARY_FILE_ACCESS));
			CloseHandle(hArcFile);
			CloseHandle(hSFXTempFile);
			DeleteFile(ArcFileName);
			DeleteFile(SFXTemporaryFileName);
			return false;
		}
		else if(Result<0){	//݃G[
			strLog.Format(IDS_ERROR_ACCESS_OUTPUT_FILE,ArcFileName);
			CloseHandle(hArcFile);
			CloseHandle(hSFXTempFile);
			DeleteFile(ArcFileName);
			DeleteFile(SFXTemporaryFileName);
			return false;
		}
		CloseHandle(hSFXTempFile);
		DeleteFile(SFXTemporaryFileName);
	}
	CloseHandle(hArcFile);

	return true;
}

bool CArchiver7ZIP::Extract(LPCTSTR ArcFileName,CConfigManager&,const CConfigExtract& Config,bool bSafeArchive,LPCTSTR OutputDir,CString &strLog)
{
	if(!IsOK()){
		return false;
	}
	if(!bSafeArchive){
		strLog.Format(IDS_ERROR_DANGEROUS_ARCHIVE,ArcFileName);
		return false;
	}

	//o͐ړ
	CCurrentDirManager currentDir(OutputDir);

	//===========================
	// DLLɓnIvV̐ݒ
	//===========================
	TRACE(_T("DLLɓnIvV̐ݒ\n"));

	CString Param;//R}hC p[^ obt@

	//𓀃p[^
	Param+=_T("x ");			//
	if(Config.ForceOverwrite){
		//㏑
		Param+=_T("-aoa ");
	}

	Param+=_T("-scsUTF-8 ");	//X|Xt@C̃R[hy[Ww

	//A[JCut@Cw
	Param+=_T("\"");
	Param+=ArcFileName;
	Param+=_T("\" ");

	ASSERT(!Param.IsEmpty());
	TRACE(_T("ArchiveHandler Commandline Parameter:%s\n"),Param);

	TRACE(_T("ArchiveHandlerĂяo\n"));
	std::vector<BYTE> szLog(LOG_BUFFER_SIZE);
	szLog[0]='\0';
	int Ret=ArchiveHandler(NULL,C2UTF8(Param),(LPSTR)&szLog[0],LOG_BUFFER_SIZE-1);
	CString strTmp;
	UtilToUNICODE(strTmp,&szLog[0],szLog.size()-1,UTILCP_UTF8);
	//strLog=&szLog[0];
	strLog=strTmp;

	return 0==Ret;
}

//X|Xt@CɃt@CGXP[vsŏށB
//Lȃt@CnhNULLłȂt@CnƁB
void CArchiver7ZIP::WriteResponceFile(HANDLE hFile,LPCTSTR fname)
{
	CPath strPath=fname;

	strPath.QuoteSpaces();

	DWORD dwWritten=0;
	//t@C+so
	std::vector<BYTE> cArray;
	UtilToUTF8(cArray,strPath+_T("\r\n"));
	WriteFile(hFile,&cArray[0],(cArray.size()-1)*sizeof(BYTE),&dwWritten,NULL);
}


//=============================================================
// SevenZipGetFileName()̏o͌ʂɁAi[ꂽt@C
// pXĂ邩ǂʂAdtH_쐬h
//=============================================================
bool CArchiver7ZIP::ExamineArchive(LPCTSTR ArcFileName,CConfigManager& ConfMan,bool bSkipDir,bool &bInFolder,bool &bSafeArchive,CString &BaseDir,CString &strErr)
{
	//UNICODESǂ肷
	ASSERT(IsOK());
	if(!IsOK()){
		return false;
	}

	if(!InspectArchiveBegin(ArcFileName,ConfMan)){
		strErr.Format(IDS_ERROR_OPEN_ARCHIVE,ArcFileName);
		return false;
	}

	bInFolder=true;
	bool bSureDir=false;	//BaseDirɓĂ镶񂪊mɃtH_łȂtrue
	TRACE(_T("========\n"));

	while(InspectArchiveNext()){
		CString Buffer;
		InspectArchiveGetFileName(Buffer);
		Buffer.Replace(_T('\\'),_T('/'));		//pX؂蕶̒u
		TRACE(_T("%s\n"),Buffer);

		/*
		Separator('/' or '\')͊i[t@C̐擪ɂ܂܂ĂĂׂł̂ŁA
		i[t@C̐擪ɂSeparatorĂtH_Ɋi[ꂽԂƂ͌ȂȂB
		SeparatorMaxRepeat葽ƕsƂ
		AMaxRepeat-1̂Ƃ̓G[Ƃ͂Ȃ
		*/
		const int Length=Buffer.GetLength();
		int StartIndex=0;
		for(;StartIndex<Length;StartIndex++){
			//擪'/'Ƃ΂Ă
#if defined(_UNICODE)||defined(UNICODE)
			if(_T('/')!=Buffer[StartIndex])break;
#else
			if(_MBC_SINGLE==_mbsbtype((const unsigned char *)(LPCTSTR)Buffer,StartIndex)){
				if(_T('/')!=Buffer[StartIndex])break;
			}
			else{	//SpȂ'/'ł͂Ȃ
				break;
			}
#endif//defined(_UNICODE)||defined(UNICODE)
		}
		if(!UtilIsSafeUnicode((LPCTSTR)Buffer+StartIndex)){	//댯UNICODEw肪΁A댯ȃt@CƌȂ
			//čI
			InspectArchiveEnd();
			bSafeArchive=false;
			bInFolder=false;
			return true;
		}

		//͓dfBNg
		//łɓdfBNg肪tĂꍇ͈Ŝ݂ɓO

		int FoundIndex=0;
		while(bInFolder){
			FoundIndex=Buffer.Find(_T('/'),StartIndex);
			if(-1==FoundIndex){	//'/'i[t@C̐擪ȊOɊ܂܂Ȃꍇ
				if(!BaseDir.IsEmpty()&&BaseDir==Buffer){
					bSureDir=true;	//BaseDirtH_łƊmFꂽ
					break;
				}
				else if(BaseDir.IsEmpty()){
					//tH_̌'/'tȂA[JCo
					//̂ŏɏoĂƂ́AtH_Ɖ肷
					BaseDir=Buffer;
					bSureDir=false;
					break;
				}
			}
			CString Dir=Buffer.Mid(StartIndex,FoundIndex-StartIndex);	//SeparatoȓO܂ł̕(fBNgɑ)𔲂oĂ
			//܂łׂ̒DirEmptył͂ȂƂۏ؂Ă
			//܂A댯ł͂ȂƂĂ
			TRACE(_T("Base=%s,Dir=%s\n"),BaseDir,Dir);

			if(_T('.')==Dir){	//./΃fBNgwƂĂ͖
				StartIndex=FoundIndex+1;
				continue;
			}
			if(BaseDir.IsEmpty()){
				BaseDir=Dir;
				bSureDir=true;
			}
			else if(BaseDir!=Dir){
				bInFolder=false;
			}
			else bSureDir=true;	//BaseDirfBNgƊmFꂽ
			break;
		}
	}
	TRACE(_T("========\n"));

	InspectArchiveEnd();
	bSafeArchive=true;

	//tH_ɓĂ悤ł͂邪AfBNgƉ肳ꂽ̏ꍇ
	if(bInFolder&&!bSureDir)bInFolder=false;
	return true;
}

bool CArchiver7ZIP::ExtractSpecifiedOnly(LPCTSTR ArcFileName,CConfigManager&,LPCTSTR OutputDir,std::list<CString> &FileList,CString &strLog,bool bUsePath)
{
	if(!IsOK()){
		return false;
	}
	//o͐ړ
	CCurrentDirManager currentDir(OutputDir);

	//==============================================
	// X|Xt@Cpe|t@C擾
	//==============================================
	TCHAR ResponceFileName[_MAX_PATH+1];
	FILL_ZERO(ResponceFileName);
	if(!UtilGetTemporaryFileName(ResponceFileName,_T("7zp"))){
		strLog=CString(MAKEINTRESOURCE(IDS_ERROR_TEMPORARY_FILE_CREATE));
		return false;
	}
	ASSERT(0!=_tcslen(ResponceFileName));

	//𓀑Ώۃt@CX|Xt@Cɏo
	{
		HANDLE hFile=CreateFile(ResponceFileName,GENERIC_WRITE,0,NULL,CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL,NULL);
		if(INVALID_HANDLE_VALUE==hFile){
			strLog=CString(MAKEINTRESOURCE(IDS_ERROR_TEMPORARY_FILE_ACCESS));
			return false;
		}

		std::list<CString>::iterator ite=FileList.begin();
		const std::list<CString>::iterator end=FileList.end();
		for(;ite!=end;ite++){
			TRACE(_T("%s=>%s\n"),LPCTSTR(*ite),OutputDir);
			WriteResponceFile(hFile,*ite);
		}
		CloseHandle(hFile);
	}

	//===========================
	// DLLɓnIvV̐ݒ
	//===========================
	CString Param;

	//𓀃p[^
	if(bUsePath){
		Param+=_T("x ");	//pXt
	}else{
		Param+=_T("e ");		//fBNgȂŉ
	}
	Param+=
		_T("-r- ")		//ċA
		_T("-scsUTF-8 ")	//X|Xt@C̃R[hy[Ww
	;

	//A[JCut@Cw
	Param+=_T("\"");
	Param+=ArcFileName;
	Param+=_T("\" ");

	//o͐w
	Param+=_T("-o\"");
	Param+=OutputDir;
	Param+=_T("\" ");

	//X|Xt@Cw
	Param+=_T("\"@");
	Param+=ResponceFileName;
	Param+=_T("\"");

	TRACE(_T("ArchiveHandlerĂяo\nCommandline Parameter:%s\n"),Param);
	std::vector<BYTE> szLog(LOG_BUFFER_SIZE);
	szLog[0]='\0';
	int Ret=ArchiveHandler(NULL,C2UTF8(Param),(LPSTR)&szLog[0],LOG_BUFFER_SIZE-1);
	CString strTmp;
	UtilToUNICODE(strTmp,&szLog[0],szLog.size()-1,UTILCP_UTF8);
	//strLog=&szLog[0];
	strLog=strTmp;
	//gX|Xt@C͏
	DeleteFile(ResponceFileName);

	return 0==Ret;
}


bool CArchiver7ZIP::DeleteItemFromArchive(LPCTSTR ArcFileName,CConfigManager&,const std::list<CString>& FileList,CString &strLog)
{
	if(!IsOK()){
		return false;
	}

	//==============================================
	// X|Xt@Cpe|t@C擾
	//==============================================
	TCHAR ResponceFileName[_MAX_PATH+1];
	FILL_ZERO(ResponceFileName);
	if(!UtilGetTemporaryFileName(ResponceFileName,_T("7zp"))){
		strLog=CString(MAKEINTRESOURCE(IDS_ERROR_TEMPORARY_FILE_CREATE));
		return false;
	}
	ASSERT(0!=_tcslen(ResponceFileName));

	//폜Ώۃt@CX|Xt@Cɏo
	{
		HANDLE hFile=CreateFile(ResponceFileName,GENERIC_WRITE,0,NULL,CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL,NULL);
		if(INVALID_HANDLE_VALUE==hFile){
			strLog=CString(MAKEINTRESOURCE(IDS_ERROR_TEMPORARY_FILE_ACCESS));
			return false;
		}

		std::list<CString>::const_iterator ite=FileList.begin();
		const std::list<CString>::const_iterator end=FileList.end();
		for(;ite!=end;ite++){
			CString tmp=*ite;
			if(tmp[tmp.GetLength()-1]==_T('\\'))tmp.Delete(tmp.GetLength()-1);
			WriteResponceFile(hFile,tmp);
		}
		CloseHandle(hFile);
	}

	//===========================
	// DLLɓnIvV̐ݒ
	//===========================
	CString Param;

	//폜p[^
	Param+=
		_T("d ")	//폜
		_T("-r- ")	//ċA
		_T("-scsUTF-8 ")	//X|Xt@C̃R[hy[Ww
	;

	//A[JCut@Cw
	Param+=_T("\"");
	Param+=ArcFileName;
	Param+=_T("\" ");

	//X|Xt@Cw
	Param+=_T("\"@");
	Param+=ResponceFileName;
	Param+=_T("\"");


	TRACE(_T("ArchiveHandlerĂяo\nCommandline Parameter:%s\n"),Param);
	//char szLog[LOG_BUFFER_SIZE]={0};
	std::vector<BYTE> szLog(LOG_BUFFER_SIZE);
	szLog[0]='\0';
	int Ret=ArchiveHandler(NULL,C2UTF8(Param),(LPSTR)&szLog[0],LOG_BUFFER_SIZE-1);
	CString strTmp;
	UtilToUNICODE(strTmp,&szLog[0],szLog.size()-1,UTILCP_UTF8);
	//strLog=&szLog[0];
	strLog=strTmp;
	//gX|Xt@C͏
	DeleteFile(ResponceFileName);

	return 0==Ret;
}

ARCRESULT CArchiver7ZIP::TestArchive(LPCTSTR ArcFileName,CString &strLog)
{
	TRACE(_T("TestArchive() called.\n"));
	ASSERT(IsOK());
	if(!IsOK()){
		return TEST_ERROR;
	}

	//tR}hɂeXgĂ
	CString Param=
		_T("t ")			//eXg
		_T("\"")
	;
	Param+=ArcFileName;
	Param+=_T("\"");

	//char szLog[LOG_BUFFER_SIZE]={0};
	std::vector<BYTE> szLog(LOG_BUFFER_SIZE);
	szLog[0]='\0';
	int Ret=ArchiveHandler(NULL,C2UTF8(Param),(LPSTR)&szLog[0],LOG_BUFFER_SIZE-1);
	CString strTmp;
	UtilToUNICODE(strTmp,&szLog[0],szLog.size()-1,UTILCP_UTF8);
	//strLog=&szLog[0];
	strLog=strTmp;



	if(Ret==0)return TEST_OK;
	else return TEST_NG;
}


//-------------------------------
//---UNICODEłI[o[Ch---
//-------------------------------
BOOL CArchiver7ZIP::CheckArchive(LPCTSTR _szFileName)
{
	if(!ArchiverCheckArchive){
		ASSERT(ArchiverCheckArchive);
		return false;
	}
	//7-Zip32.dlĺAwb_Ít@CCheckArchiveAEBhEnh݂ȂƎs
	CDummyWindow dummy;
	dummy.Create(NULL,CWindow::rcDefault);
	BOOL bRet=ArchiverCheckArchive(C2UTF8(_szFileName),CHECKARCHIVE_BASIC);
	dummy.DestroyWindow();
	return bRet;
}

int CArchiver7ZIP::GetFileCount(LPCTSTR _szFileName)
{
	if(!ArchiverGetFileCount){
		return -1;
	}
	return ArchiverGetFileCount(C2UTF8(_szFileName));
}


bool CArchiver7ZIP::InspectArchiveBegin(LPCTSTR ArcFileName,CConfigManager&)
{
	TRACE(_T("CArchiverDLL::InspectArchiveBegin()\n"));
	ASSERT(ArchiverOpenArchive);
	if(!ArchiverOpenArchive){
		return false;
	}
	if(m_hInspectArchive){
		ASSERT(!"Close the Archive First!!!\n");
		return false;
	}
	m_hInspectArchive=ArchiverOpenArchive(NULL,C2UTF8(ArcFileName),m_dwInspectMode);
	if(!m_hInspectArchive){
		TRACE(_T("Failed to Open Archive\n"));
		return false;
	}
	m_bInspectFirstTime=true;

	FILL_ZERO(m_IndividualInfo);
	return true;
}

bool CArchiver7ZIP::InspectArchiveGetFileName(CString &FileName)
{
	if(ArchiverGetFileName){
		if(!m_hInspectArchive){
			ASSERT(!"Open an Archive First!!!\n");
			return false;
		}
		std::vector<BYTE> szBuffer(FNAME_MAX32*2+1);
		ArchiverGetFileName(m_hInspectArchive,(LPCSTR)&szBuffer[0],FNAME_MAX32*2);
		UtilToUNICODE(FileName,&szBuffer[0],szBuffer.size(),UTILCP_UTF8);
		return true;
	}
	else{
		ASSERT(LOAD_DLL_STANDARD!=m_LoadLevel);
		if(!m_hInspectArchive){
			ASSERT(!"Open an Archive First!!!\n");
			return false;
		}
		UtilToUNICODE(FileName,(LPCBYTE)m_IndividualInfo.szFileName,sizeof(m_IndividualInfo.szFileName),UTILCP_UTF8);
		return true;
	}
}

bool CArchiver7ZIP::InspectArchiveGetMethodString(CString &strMethod)
{
	if(!m_hInspectArchive){
		ASSERT(!"Open an Archive First!!!\n");
		return false;
	}

	//\0ŏIĂ邩ǂۏ؂łȂ
	char szBuffer[32]={0};

	if(ArchiverGetMethod){
		if(0!=ArchiverGetMethod(m_hInspectArchive,szBuffer,31)){
			//\̂擾
			strncpy_s(szBuffer,m_IndividualInfo.szMode,8);
		}
	}
	else{
		//\̂擾
		strncpy_s(szBuffer,m_IndividualInfo.szMode,8);
	}

	//i[
	UtilToUNICODE(strMethod,(LPCBYTE)szBuffer,9,UTILCP_UTF8);
	return true;
}

bool CArchiver7ZIP::AddItemToArchive(LPCTSTR ArcFileName,const std::list<CString> &FileList,CConfigManager &ConfMan,LPCTSTR lpDestDir,CString &strLog)
{
	// X|Xt@Cpe|t@C擾
	TCHAR ResponceFileName[_MAX_PATH+1];
	FILL_ZERO(ResponceFileName);
	if(!UtilGetTemporaryFileName(ResponceFileName,_T("zip"))){
		strLog=CString(MAKEINTRESOURCE(IDS_ERROR_TEMPORARY_FILE_CREATE));
		return false;
	}
	ASSERT(0!=_tcslen(ResponceFileName));

	//===ꎞIɃt@CRs[
	//---\ŏI_pX擾
	CPath strBasePath;
	UtilGetBaseDirectory(strBasePath,FileList);
	TRACE(_T("%s\n"),strBasePath);

	//---e|ɑΏۃt@CRs[
	//e|
	CTemporaryDirectoryManager tdm(_T("lhaf"));
	CPath strDestPath(tdm.GetDirPath());
	strDestPath+=lpDestDir;
	UtilMakeSureDirectoryPathExists(strDestPath);

	// kΏۃt@CC
	const int BasePathLength=((CString)strBasePath).GetLength();
	CString strSrcFiles;	//Rs[t@C̈ꗗ
	CString strDestFiles;	//Rs[t@C̈ꗗ
	std::list<CString>::const_iterator ite;
	for(ite=FileList.begin();ite!=FileList.end();++ite){
		//x[XpXɑ΃pX擾 : ʂłpX̕Jbg
		LPCTSTR lpSrc((LPCTSTR)(*ite)+BasePathLength);

		//葤t@Cw
		strSrcFiles+=(strBasePath+lpSrc);	//PathAppend
		strSrcFiles+=_T('|');
		//󂯑t@Cw
		strDestFiles+=strDestPath+lpSrc;
		strDestFiles+=_T('|');
	}
	strSrcFiles+=_T('|');
	strDestFiles+=_T('|');

	//'|''\0'ɕϊ
	std::vector<TCHAR> srcBuf(strSrcFiles.GetLength()+1);
	UtilMakeFilterString(strSrcFiles,&srcBuf[0],srcBuf.size());
	std::vector<TCHAR> destBuf(strDestFiles.GetLength()+1);
	UtilMakeFilterString(strDestFiles,&destBuf[0],destBuf.size());

	//t@Ce
	SHFILEOPSTRUCT fileOp={0};
	fileOp.wFunc=FO_COPY;
	fileOp.fFlags=FOF_MULTIDESTFILES|FOF_NOCONFIRMATION|FOF_NOCONFIRMMKDIR|FOF_NOCOPYSECURITYATTRIBS|FOF_NO_CONNECTED_ELEMENTS;
	fileOp.pFrom=&srcBuf[0];
	fileOp.pTo=&destBuf[0];

	//Rs[s
	if(::SHFileOperation(&fileOp)){
		//G[
		strLog=CString(MAKEINTRESOURCE(IDS_ERROR_FILE_COPY));
		return false;
	}else if(fileOp.fAnyOperationsAborted){
		//LZ
		strLog=CString(MAKEINTRESOURCE(IDS_ERROR_USERCANCEL));
		return false;
	}

	//JgfBNgݒ
	::SetCurrentDirectory(tdm.GetDirPath());
	// ɁAX|Xt@CɃA[JCuшkΏۃt@CL
	{
		HANDLE hFile=CreateFile(ResponceFileName,GENERIC_WRITE,0,NULL,CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL,NULL);
		if(INVALID_HANDLE_VALUE==hFile){
			strLog=CString(MAKEINTRESOURCE(IDS_ERROR_TEMPORARY_FILE_ACCESS));
			return false;
		}
		//X|Xt@Cւ̏
		//SĈk
		WriteResponceFile(hFile,_T("*"));
		CloseHandle(hFile);
	}


	//===========================
	// DLLɓnIvV̐ݒ
	//===========================
	TRACE(_T("DLLɓnIvV̐ݒ\n"));

	CString Param;//R}hC p[^ obt@

	CConfigZIP confZIP;
	CConfig7Z  conf7Z;
	ASSERT(ArchiverGetArchiveType);
	switch(ArchiverGetArchiveType(C2UTF8(ArcFileName))){
	case 1:	//ZIP`ňk
		confZIP.load(ConfMan);
		if(!FormatCompressCommandZIP(confZIP,Param,false,0,NULL,NULL,strLog)){
			DeleteFile(ResponceFileName);
			return false;
		}
		break;
	case 2:	//7z`ňk
		conf7Z.load(ConfMan);
		if(!FormatCompressCommand7Z(conf7Z,Param,0,NULL,strLog)){
			DeleteFile(ResponceFileName);
			return false;
		}
		break;
	default:
		ASSERT(!"This code cannot be run");
		//G[ʓ|Ȃ̂ŕĂB
		return false;
	}

	Param+=_T("-scsUTF-8 ");	//X|Xt@C̃R[hy[Ww

	//kt@Cw
	Param+=_T("\"");
	Param+=ArcFileName;
	Param+=_T("\" ");

	//X|Xt@Cw
	Param+=_T("\"@");
	Param+=ResponceFileName;
	Param+=_T("\"");

	ASSERT(!Param.IsEmpty());
	TRACE(_T("ArchiveHandler Commandline Parameter:%s\n"),Param);

	TRACE(_T("ArchiveHandlerĂяo\n"));
	//char szLog[LOG_BUFFER_SIZE]={0};
	std::vector<BYTE> szLog(LOG_BUFFER_SIZE);
	szLog[0]='\0';
	int Ret=ArchiveHandler(NULL,C2UTF8(Param),(LPSTR)&szLog[0],LOG_BUFFER_SIZE-1);
	CString strTmp;
	UtilToUNICODE(strTmp,&szLog[0],szLog.size()-1,UTILCP_UTF8);
	//strLog=&szLog[0];
	strLog=strTmp;

	//gX|Xt@C͏
	DeleteFile(ResponceFileName);

	return 0==Ret;
}


//w肳ꂽfBNgWJ;
bool CArchiver7ZIP::ExtractDirectoryEntry(LPCTSTR lpszArcFile,CConfigManager &ConfMan,const ARCHIVE_ENTRY_INFO_TREE* lpBase,const ARCHIVE_ENTRY_INFO_TREE* lpDir,LPCTSTR lpszOutputBaseDir,bool bCollapseDir,CString &strLog)
{
	//---ꎞtH_ɂ܂Ƃ߂ēWJAォtH_\؂o
	std::list<CString> files;

	CString strPath;

	bool bRestoreDir=false;
	if(lpDir->strFullPath.IsEmpty()){
		//fBNgo^ĂȂ̂ŃpXZo
		ArcEntryInfoTree_GetNodePathRelative(lpDir,lpBase,strPath);
		strPath.Replace(_T('/'),_T('\\'));

		CPath tmpPath(strPath);
		tmpPath.RemoveBackslash();	//fBNg症ɂ
		tmpPath.RemoveFileSpec();	//efBNg܂Ő؂߂
		tmpPath.Append(_T("*"));	//fBNgȉ̑SẴt@CWJ
		files.push_back(tmpPath);

		bRestoreDir=true;
	}else{
		//fBNgA[JCuɃGgƂēo^ĂȂo͂
		CString tmpPath(lpDir->strFullPath);
		if(tmpPath[tmpPath.GetLength()-1]==_T('\\')||tmpPath[tmpPath.GetLength()-1]==_T('/')){
			//\/폜
			tmpPath.Delete(tmpPath.GetLength()-1);
		}
		files.push_back(tmpPath);

		strPath=lpDir->strFullPath;
		strPath.Replace(_T('/'),_T('\\'));
	}

	//--------------------------------------
	// Cꂽo̓fBNgpXZo
	//--------------------------------------
	//---{̏o͐
	CString strOutputDir=lpszOutputBaseDir+strPath;
	if(strOutputDir.GetLength()>_MAX_PATH){
		//tH_Ȃ肷
		strLog=CString(MAKEINTRESOURCE(IDS_ERROR_MAX_PATH));
		return false;
	}

	//FileListɂ͂PĂȂ͂
	ASSERT(files.size()==1);

	//ꎞtH_
	CTemporaryDirectoryManager tdm(_T("lhaf"));
	CPath strTempOutput(tdm.GetDirPath());
	if(bRestoreDir){	//fBNgƂŕ
		strTempOutput+=strPath;
		strTempOutput.AddBackslash();
		TRACE(_T("Ƃł̃fBNg:%s\n"),(LPCTSTR)strTempOutput);
		if(!UtilMakeSureDirectoryPathExists(strTempOutput)){
			strLog.Format(IDS_ERROR_CANNOT_MAKE_DIR,strTempOutput);
			return false;
		}
	}
	//---ꎞtH_ɂ܂Ƃ߂ēWJAォtH_\؂o
	// t@CWJ
	if(!ExtractSpecifiedOnly(lpszArcFile,ConfMan,strTempOutput,files,strLog,true)){
		return false;
	}

	//葤t@Cw
	CPath tmp(strTempOutput);
	tmp+=(LPCTSTR)strPath;	//PathAppend
	tmp.RemoveBackslash();
	CString strSrcFiles(tmp);
	strSrcFiles+=_T("||");
	//󂯑t@Cw
	tmp=lpszOutputBaseDir;
	{
		CString strTmp;
		ArcEntryInfoTree_GetNodePathRelative(lpDir,lpBase,strTmp);
		strTmp.Replace(_T('/'),_T('\\'));
		tmp+=(LPCTSTR)strTmp;
	}
	tmp.AddBackslash();
	CString strDestFiles(tmp);
	strDestFiles+=_T("||");

	//'|''\0'ɕϊ
	std::vector<TCHAR> srcBuf(strSrcFiles.GetLength()+1);
	UtilMakeFilterString(strSrcFiles,&srcBuf[0],srcBuf.size());
	std::vector<TCHAR> destBuf(strDestFiles.GetLength()+1);
	UtilMakeFilterString(strDestFiles,&destBuf[0],destBuf.size());

	//t@Ce
	SHFILEOPSTRUCT fileOp={0};
	fileOp.wFunc=FO_MOVE;
	fileOp.fFlags=FOF_MULTIDESTFILES|/*FOF_NOCONFIRMATION|*/FOF_NOCONFIRMMKDIR|FOF_NOCOPYSECURITYATTRIBS|FOF_NO_CONNECTED_ELEMENTS;
	fileOp.pFrom=&srcBuf[0];
	fileOp.pTo=&destBuf[0];

	//ړs
	if(::SHFileOperation(&fileOp)){
		//G[
		strLog=CString(MAKEINTRESOURCE(IDS_ERROR_FILE_MOVE));
		return false;
	}else if(fileOp.fAnyOperationsAborted){
		//LZ
		strLog=CString(MAKEINTRESOURCE(IDS_ERROR_USERCANCEL));
		return false;
	}

	return true;

}

