/*
 * Copyright (c) 2005-2012, Claybird
 * All rights reserved.

 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:

 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the Claybird nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#include "stdafx.h"
#include "ArchiverUNHKI.h"
#include "../Utilities/FileOperation.h"
#include "../Utilities/OSUtil.h"
#include "../resource.h"
#include "../ConfigCode/ConfigExtract.h"
#include "../ConfigCode/ConfigUNHKI.h"

CArchiverUNHKI::CArchiverUNHKI()
{
	m_nRequiredVersion=3;
	m_nRequiredSubVersion=19;
	m_strDllName=_T("UnHki32.DLL");
	m_AstrPrefix="UnHki";
}

CArchiverUNHKI::~CArchiverUNHKI()
{
	FreeDLL();
}

bool CArchiverUNHKI::Compress(LPCTSTR ArcFileName,std::list<CString> &ParamList,CConfigManager &ConfMan,const PARAMETER_TYPE,int Options,LPCTSTR,LPCTSTR,LPCTSTR lpszLevel,CString &strLog)
{
	if(!IsOK()){
		return false;
	}

	ASSERT(0!=_tcslen(ArcFileName));
	TRACE(_T("ArcFileName=%s\n"),ArcFileName);

	//==============================================
	// X|Xt@Cpe|t@C擾
	//==============================================
	TCHAR ResponceFileName[_MAX_PATH+1];
	FILL_ZERO(ResponceFileName);
	if(!UtilGetTemporaryFileName(ResponceFileName,_T("hki"))){
		strLog=CString(MAKEINTRESOURCE(IDS_ERROR_TEMPORARY_FILE_CREATE));
		return false;
	}
	ASSERT(0!=_tcslen(ResponceFileName));

	//============================================
	// ȉ𓀃t@Cpe|t@C擾
	//============================================
	TCHAR SFXTemporaryFileName[_MAX_PATH+1];
	FILL_ZERO(SFXTemporaryFileName);
	bool bSFX=(0!=(Options&COMPRESS_SFX));
	if(bSFX){
		//2iK쐬
		if(!UtilGetTemporaryFileName(SFXTemporaryFileName,_T("sfx"))){
			strLog=CString(MAKEINTRESOURCE(IDS_ERROR_TEMPORARY_FILE_CREATE));
			return false;
		}
		ASSERT(0!=_tcslen(SFXTemporaryFileName));
	}

	//==========================================
	// X|Xt@CɃA[JCu
	// kΏۃt@CL
	//==========================================
	{
		HANDLE hFile=CreateFile(ResponceFileName,GENERIC_WRITE,0,NULL,CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL,NULL);
		if(INVALID_HANDLE_VALUE==hFile){
			strLog=CString(MAKEINTRESOURCE(IDS_ERROR_TEMPORARY_FILE_ACCESS));
			return false;
		}

		TRACE(_T("X|Xt@Cւ̏\n"));

		std::list<CString>::iterator ite;
		for(ite=ParamList.begin();ite!=ParamList.end();ite++){
			WriteResponceFile(hFile,*ite);
		}
		CloseHandle(hFile);
	}

	//===========================
	// DLLɓnIvV̐ݒ
	//===========================
	TRACE(_T("DLLɓnIvV̐ݒ\n"));

	CString Param;//R}hC p[^ obt@

	//kp[^
	Param+=
		_T("a ")	//k
		_T("-a1 ")	//ۑ
		_T("-r0 ")	//ċAI
		_T("-jf0 ")	//[gL폜
	;

	CConfigHKI Config;
	Config.load(ConfMan);
	//pX[h
	if(Options&COMPRESS_PASSWORD){
		Param+=_T("-mp ");
		switch(Config.EncryptAlgorithm){
		case HKI_ENCRYPT_NONE:
			Param+=_T("-me0 ");
			break;
		case HKI_ENCRYPT_RIJNDAEL128:
			Param+=_T("-me1 ");
			break;
		case HKI_ENCRYPT_RIJNDAEL256:
			Param+=_T("-me2 ");
			break;
		case HKI_ENCRYPT_SINGLE_DES:
			Param+=_T("-me3 ");
			break;
		case HKI_ENCRYPT_TRIPLE_DES:
			Param+=_T("-me4 ");
			break;
		case HKI_ENCRYPT_BLOWFISH448:
			Param+=_T("-me5 ");
			break;
		case HKI_ENCRYPT_TWOFISH128:
			Param+=_T("-me6 ");
			break;
		case HKI_ENCRYPT_TWOFISH256:
			Param+=_T("-me7 ");
			break;
		case HKI_ENCRYPT_SQUARE:
			Param+=_T("-me8 ");
			break;
		}
	}

	//kx
	if(lpszLevel && *lpszLevel!=_T('\0')){
		Param.AppendFormat(_T("-ml%s "),lpszLevel);
	}else{
		switch(Config.CompressLevel){
		case HKI_COMPRESS_LEVEL_NONE:
			Param+=_T("-ml0 ");
			break;
		case HKI_COMPRESS_LEVEL_FAST:
			Param+=_T("-ml1 ");
			break;
		case HKI_COMPRESS_LEVEL_NORMAL:
			Param+=_T("-ml2 ");
			break;
		case HKI_COMPRESS_LEVEL_HIGH:
			Param+=_T("-ml3 ");
			break;
		}
	}

	//kt@C̐ݒ
	if(bSFX){
		Param+=_T("\"");
		Param+=SFXTemporaryFileName;
		Param+=_T("\" ");
	}else{
		Param+=_T("\"");
		Param+=ArcFileName;
		Param+=_T("\" ");
	}
 
	//X|Xt@Cw
	Param+=_T("\"@");
	Param+=ResponceFileName;
	Param+=_T("\"");

	ASSERT(!Param.IsEmpty());
	TRACE(_T("ArchiveHandler Commandline Parameter:%s\n"),Param);

	TRACE(_T("ArchiveHandlerĂяo\n"));
	std::vector<char> szLog(LOG_BUFFER_SIZE);
	szLog[0]='\0';
	int Ret=ArchiveHandler(NULL,CT2A(Param),&szLog[0],LOG_BUFFER_SIZE-1);
	strLog=&szLog[0];

	//gX|Xt@C͏
	DeleteFile(ResponceFileName);

	//G[
	if(!bSFX||0!=Ret){
		if(bSFX){
			DeleteFile(SFXTemporaryFileName);
		}
		return 0==Ret;
	}

	//====================
	// ȉ𓀏ɂɕϊ
	//====================
	//ϊp[^
	Param=_T("s ");		//ϊ

	//SFXW[pX
	TCHAR SFXModulePath[_MAX_PATH+1];
	FILL_ZERO(SFXModulePath);
	LPTSTR lptemp;
	{
		//SȃpXɈړ;DLLǂݍݑ΍
		CCurrentDirManager cdm(UtilGetModuleDirectoryPath());
		if(!SearchPath(NULL,_T("HKI.SFX"),NULL,_MAX_PATH,SFXModulePath,&lptemp)){
			strLog.Format(IDS_ERROR_SFX_MODULE_NOT_FOUND,_T("HKI.SFX"));
			DeleteFile(SFXTemporaryFileName);
			return false;
		}
	}
	PathQuoteSpaces(SFXModulePath);
	Param+=_T("-mx");
	Param+=SFXModulePath;
	Param+=_T(" ");

	Param+=("-gr\"");	//ϊt@C
	Param+=ArcFileName;
	Param+=_T("\" \"");
	Param+=SFXTemporaryFileName;	//ϊt@C
	Param+=_T("\" ");


	ASSERT(!Param.IsEmpty());
	TRACE(_T("ArchiveHandler Commandline Parameter(SFX):%s\n"),Param);

	TRACE(_T("SFXpArchiveHandlerĂяo\n"));
	std::vector<char> LogSFX(LOG_BUFFER_SIZE);
	LogSFX[0]='\0';
	Ret=ArchiveHandler(NULL,CT2A(Param),&LogSFX[0],LOG_BUFFER_SIZE-1);
	strLog+=&LogSFX[0];

	DeleteFile(SFXTemporaryFileName);

	return 0==Ret;
}

//bSafeArchive͖
bool CArchiverUNHKI::Extract(LPCTSTR ArcFileName,CConfigManager&,const CConfigExtract &Config,bool,LPCTSTR OutputDir,CString &strLog)
{
	if(!IsOK()){
		return false;
	}

	//o͐ړ
	CCurrentDirManager currentDir(OutputDir);
	//===========================
	// DLLɓnIvV̐ݒ
	//===========================
	TRACE(_T("DLLɓnIvV̐ݒ\n"));

	CString Param;//R}hC p[^ obt@

	//𓀃p[^
	Param+=
		_T("x ")			//
		_T("-a1 ")			//ۑ
//		_T("-r1 ")			//ċAI
		_T("-x1 ")			//pXێ
	;
	if(Config.ForceOverwrite){
		//㏑
		Param+=_T("-gf1 ");
	}else{
		Param+=_T("-gf0 ");
	}

	//A[JCut@Cw
	Param+=_T("\"");
	Param+=ArcFileName;
	Param+=_T("\" ");

	ASSERT(!Param.IsEmpty());
	TRACE(_T("ArchiveHandler Commandline Parameter:%s\n"),Param);

	TRACE(_T("ArchiveHandlerĂяo\n"));
	std::vector<char> szLog(LOG_BUFFER_SIZE);
	szLog[0]='\0';
	int Ret=ArchiveHandler(NULL,CT2A(Param),&szLog[0],LOG_BUFFER_SIZE-1);
	strLog=&szLog[0];

	return 0==Ret;
}


//=========================================================
// UnHkiGetFileName()̏o͌ʂɁAi[ꂽt@CpX
// Ă邩ǂʂAdtH_쐬h
//=========================================================
bool CArchiverUNHKI::ExamineArchive(LPCTSTR ArcFileName,CConfigManager& ConfMan,bool bSkipDir,bool &bInFolder,bool &bSafeArchive,CString &BaseDir,CString &strErr)
{
	if(bSkipDir){
		TRACE(_T("dtH_\n"));
		return true;
	}
	return _ExamineArchiveFast(ArcFileName,ConfMan,bInFolder,BaseDir,strErr);
}

BOOL CArchiverUNHKI::CheckArchive(LPCTSTR _szFileName)
{
	if(!ArchiverCheckArchive){
		ASSERT(ArchiverCheckArchive);
		return false;
	}
	return ArchiverCheckArchive(CT2A(_szFileName),CHECKARCHIVE_RAPID);
}

bool CArchiverUNHKI::InspectArchiveGetWriteTime(FILETIME &FileTime)
{
	if(!m_hInspectArchive){
		ASSERT(!"Open an Archive First!!!\n");
		return false;
	}
	//gŊ֐Ŏ擾
	if(ArchiverGetWriteTimeEx){
		if(!ArchiverGetWriteTimeEx(m_hInspectArchive,&FileTime))return false;
		return true;
	}
	else{
		//INDIVIDUALINFO玞擾
		FILETIME TempTime;
		if(!DosDateTimeToFileTime(m_IndividualInfo.wDate,m_IndividualInfo.wTime,&TempTime))return false;
		if(!LocalFileTimeToFileTime(&TempTime,&FileTime))return false;
		return true;
	}
}

bool CArchiverUNHKI::ExtractSpecifiedOnly(LPCTSTR ArcFileName,CConfigManager&,LPCTSTR OutputDir,std::list<CString> &FileList,CString &strLog,bool bUsePath)
{
	if(!IsOK()){
		return false;
	}

	//o͐ړ
	CCurrentDirManager currentDir(OutputDir);
	//==============================================
	// X|Xt@Cpe|t@C擾
	//==============================================
	TCHAR ResponceFileName[_MAX_PATH+1];
	FILL_ZERO(ResponceFileName);
	if(!UtilGetTemporaryFileName(ResponceFileName,_T("hki"))){
		strLog=CString(MAKEINTRESOURCE(IDS_ERROR_TEMPORARY_FILE_CREATE));
		return false;
	}
	ASSERT(0!=_tcslen(ResponceFileName));

	//𓀑Ώۃt@CX|Xt@Cɏo
	{
		HANDLE hFile=CreateFile(ResponceFileName,GENERIC_WRITE,0,NULL,CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL,NULL);
		if(INVALID_HANDLE_VALUE==hFile){
			strLog=CString(MAKEINTRESOURCE(IDS_ERROR_TEMPORARY_FILE_ACCESS));
			return false;
		}

		std::list<CString>::iterator ite=FileList.begin();
		const std::list<CString>::iterator end=FileList.end();
		for(;ite!=end;ite++){
			WriteResponceFile(hFile,*ite);
		}
		CloseHandle(hFile);
	}

	//===========================
	// DLLɓnIvV̐ݒ
	//===========================
	CString Param;

	//𓀃p[^
	if(bUsePath){
		Param+=_T("x ");				//pX
	}else{
		Param+=_T("e ");			//pX
	}
	Param+=
		_T("-a1 ")			//ۑ
//		_T("-r1 ")			//ċAI
//		_T("-x1 ")			//pXێ
		_T("-gf0 ")			//㏑mF
	;

	//A[JCut@Cw
	Param+=_T("\"");
	Param+=ArcFileName;
	Param+=_T("\" ");

	//o͐tH_
	Param+=_T("\"");
	Param+=OutputDir;
	Param+=_T("\" ");

	//X|Xt@Cw
	Param+=_T("\"@");
	Param+=ResponceFileName;
	Param+=_T("\"");

	//o͐fBNg쐬Ă
	if(!UtilMakeSureDirectoryPathExists(OutputDir)){
		strLog.Format(IDS_ERROR_CANNOT_MAKE_DIR,OutputDir);
		return false;
	}

	TRACE(_T("ArchiveHandlerĂяo\nCommandline Parameter:%s\n"),Param);
	//char szLog[LOG_BUFFER_SIZE]={0};
	std::vector<char> szLog(LOG_BUFFER_SIZE);
	szLog[0]='\0';
	int Ret=ArchiveHandler(NULL,CT2A(Param),&szLog[0],LOG_BUFFER_SIZE-1);
	strLog=&szLog[0];
	//gX|Xt@C͏
	DeleteFile(ResponceFileName);

	return 0==Ret;
}

bool CArchiverUNHKI::DeleteItemFromArchive(LPCTSTR ArcFileName,CConfigManager&,const std::list<CString>& FileList,CString &strLog)
{
	if(!IsOK()){
		return false;
	}

	//==============================================
	// X|Xt@Cpe|t@C擾
	//==============================================
	TCHAR ResponceFileName[_MAX_PATH+1];
	FILL_ZERO(ResponceFileName);
	if(!UtilGetTemporaryFileName(ResponceFileName,_T("hki"))){
		strLog=CString(MAKEINTRESOURCE(IDS_ERROR_TEMPORARY_FILE_CREATE));
		return false;
	}
	ASSERT(0!=_tcslen(ResponceFileName));

	//폜Ώۃt@CX|Xt@Cɏo
	{
		HANDLE hFile=CreateFile(ResponceFileName,GENERIC_WRITE,0,NULL,CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL,NULL);
		if(INVALID_HANDLE_VALUE==hFile){
			strLog=CString(MAKEINTRESOURCE(IDS_ERROR_TEMPORARY_FILE_ACCESS));
			return false;
		}

		std::list<CString>::const_iterator ite=FileList.begin();
		const std::list<CString>::const_iterator end=FileList.end();
		for(;ite!=end;ite++){
			WriteResponceFile(hFile,*ite);
		}
		CloseHandle(hFile);
	}

	//===========================
	// DLLɓnIvV̐ݒ
	//===========================
	CString Param;

	//폜p[^
	Param+=_T("d ");		//폜

	//A[JCut@Cw
	Param+=_T("\"");
	Param+=ArcFileName;
	Param+=_T("\" ");

	//X|Xt@Cw
	Param+=_T("\"@");
	Param+=ResponceFileName;
	Param+=_T("\"");

	TRACE(_T("ArchiveHandlerĂяo\nCommandline Parameter:%s\n"),Param);
	//char szLog[LOG_BUFFER_SIZE]={0};
	std::vector<char> szLog(LOG_BUFFER_SIZE);
	szLog[0]='\0';
	int Ret=ArchiveHandler(NULL,CT2A(Param),&szLog[0],LOG_BUFFER_SIZE-1);
	strLog=&szLog[0];
	//gX|Xt@C͏
	DeleteFile(ResponceFileName);

	return 0==Ret;
}

ARCRESULT CArchiverUNHKI::TestArchive(LPCTSTR ArcFileName,CString &strLog)
{
	TRACE(_T("TestArchive() called.\n"));
	ASSERT(IsOK());
	if(!IsOK()){
		return TEST_ERROR;
	}

	//tR}hɂeXgĂ
	CString Param=
		_T("t ")			//eXg
		_T("-mt2 ")			//[h:SCRC`FbN(CHECKARCHIVE_FULLCRC)

		_T("\"")
	;
	Param+=ArcFileName;
	Param+=_T("\"");

	//t@COɒǉ
	strLog=ArcFileName;
	strLog+=_T("\r\n\r\n");
	//char szLog[LOG_BUFFER_SIZE]={0};
	std::vector<char> szLog(LOG_BUFFER_SIZE);
	szLog[0]='\0';
	int Ret=ArchiveHandler(NULL,CT2A(Param),&szLog[0],LOG_BUFFER_SIZE-1);
	strLog=&szLog[0];

	strLog+=_T("\r\n");
	if(0==Ret){
		strLog+=CString(MAKEINTRESOURCE(IDS_TESTARCHIVE_RESULT_OK));
		return TEST_OK;
	}
	else{
		strLog+=CString(MAKEINTRESOURCE(IDS_TESTARCHIVE_RESULT_NG));
		return TEST_NG;
	}
}
