/*
 * Copyright (c) 2005-2012, Claybird
 * All rights reserved.

 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:

 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the Claybird nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#include "stdafx.h"
#include "ConfigCode/ConfigManager.h"
#include "CmdLineInfo.h"
#include "ArchiverManager.h"
#include "Utilities/FileOperation.h"
#include "Utilities/OSUtil.h"
#include "Utilities/StringUtil.h"
#include "compress.h"
#include "extract.h"
#include "ConfigCode/ConfigOpenAction.h"
#include "ConfigCode/ConfigGeneral.h"


CMDLINEINFO::CMDLINEINFO():
	CompressType(PARAMETER_UNDEFINED),
	Options(0),
	bSingleCompression(false),
	idForceDLL(DLL_ID_UNKNOWN),
	OutputToOverride((OUTPUT_TO)-1),
	CreateDirOverride((CREATE_OUTPUT_DIR)-1),
	IgnoreTopDirOverride(-1),
	DeleteAfterProcess(-1),
	PriorityOverride(LFPRIOTITY_DEFAULT)
{}


class COpenActionDialog : public CDialogImpl<COpenActionDialog>
{
public:
	enum {IDD = IDD_DIALOG_OPENACTION_SELECT};
	// bZ[W}bv
	BEGIN_MSG_MAP_EX(COpenActionDialog)
		COMMAND_ID_HANDLER_EX(IDC_BUTTON_OPENACTION_EXTRACT, OnButton)
		COMMAND_ID_HANDLER_EX(IDC_BUTTON_OPENACTION_LIST, OnButton)
		COMMAND_ID_HANDLER_EX(IDC_BUTTON_OPENACTION_TEST, OnButton)
		COMMAND_ID_HANDLER_EX(IDCANCEL, OnButton)
	END_MSG_MAP()

	void OnButton(UINT uNotifyCode, int nID, HWND hWndCtl){
		EndDialog(nID);
	}
};


//JI
PROCESS_MODE SelectOpenAction()
{
	COpenActionDialog Dialog;
	switch(Dialog.DoModal()){
	case IDC_BUTTON_OPENACTION_EXTRACT:
		return PROCESS_EXTRACT;
	case IDC_BUTTON_OPENACTION_LIST:
		return PROCESS_LIST;
	case IDC_BUTTON_OPENACTION_TEST:
		return PROCESS_TEST;
	default:
		return PROCESS_INVALID;
	}
}

//-----------

//R}hC߂t@C̏@肷
PROCESS_MODE ParseCommandLine(CConfigManager &ConfigManager,CMDLINEINFO &cli)
{
	std::vector<CString> ParamsArray;
	int nArgc=UtilGetCommandLineParams(ParamsArray);
#if defined(_DEBUG)
	//For Debug
//	MessageBox(NULL,GetCommandLine(),_T("CommandLine"),MB_OK|MB_ICONINFORMATION);
	TRACE(_T("---Command Parameter Dump---\n"));
	for(int i=0;i<nArgc;i++)
	{
		TRACE(_T("ParamsArray[%d]=%s\n"),i,ParamsArray[i]);
	}
	TRACE(_T("---End Dump---\n\n"));
#endif

	const bool bPressedShift=GetKeyState(VK_SHIFT)<0;	//SHIFTĂ邩ǂ
	const bool bPressedControl=GetKeyState(VK_CONTROL)<0;	//CONTROLĂ邩ǂ
	PROCESS_MODE ProcessMode=PROCESS_AUTOMATIC;

	//ftHglǂݍ
	CString strErr;
	if(!ConfigManager.LoadConfig(strErr))ErrorMessage(strErr);

	UTIL_CODEPAGE uCodePage=UTILCP_SJIS;	//X|Xt@C̃R[hy[Ww

	for(int iIndex=1;iIndex<nArgc;iIndex++){
		if(0!=_tcsncmp(_T("/"),ParamsArray[iIndex],1)){//IvVł͂Ȃ
			//t@CƂ݂ȂAΏۃt@C̃Xgɋlߍ
			//ȉ̓t@C̏
			if(0>=_tcslen(ParamsArray[iIndex])){	//NULLȂ疳
				continue;
			}
			cli.FileList.push_back(ParamsArray[iIndex]);
		}else{
			//------------------
			// IvV̉
			//------------------
			CString Parameter(ParamsArray[iIndex]);
			//ɕϊ
			Parameter.MakeLower();
			if(0==_tcsncmp(_T("/cfg"),Parameter,4)){//ݒt@Cw
				if(0==_tcsncmp(_T("/cfg:"),Parameter,5)){
					//o̓t@C̐؂o;̎_""͊OĂ
					cli.ConfigPath=(LPCTSTR)ParamsArray[iIndex]+5;

					//---ϐ(LhaForgeƎ`ϐ)WJ
					//p[^WJɕKvȏ
					std::map<stdString,CString> envInfo;
					UtilMakeExpandInformation(envInfo);

					//R}hEp[^WJ
					UtilExpandTemplateString(cli.ConfigPath, cli.ConfigPath, envInfo);
				}else if(_T("/cfg")==Parameter){
					cli.ConfigPath.Empty();
				}else{
					CString msg;
					msg.Format(IDS_ERROR_INVALID_PARAMETER,ParamsArray[iIndex]);
					ErrorMessage(msg);
					return PROCESS_INVALID;
				}
				//ݒt@C̎w肪L΃Zbg;΃ftHgɖ߂
				if(cli.ConfigPath.IsEmpty()){
					ConfigManager.SetConfigFile(NULL);
				}else{
					ConfigManager.SetConfigFile(cli.ConfigPath);
				}
				//ύX̒lǂݍ
				if(!ConfigManager.LoadConfig(strErr))ErrorMessage(strErr);
				TRACE(_T("ConfigPath=%s\n"),cli.ConfigPath);
			}else if(0==_tcsncmp(_T("/cp"),Parameter,3)){//X|Xt@C̃R[hy[Ww
				if(0==_tcsncmp(_T("/cp:"),Parameter,4)){
					CString cp((LPCTSTR)Parameter+4);
					cp.MakeLower();
					if(cp==_T("utf8")||cp==_T("utf-8")){
						uCodePage=UTILCP_UTF8;
					}else if(cp==_T("utf16")||cp==_T("utf-16")||cp==_T("unicode")){
						uCodePage=UTILCP_UTF16;
					}else if(cp==_T("sjis")||cp==_T("shiftjis")||cp==_T("s-jis")||cp==_T("s_jis")){
						uCodePage=UTILCP_SJIS;
					}else{
						CString msg;
						msg.Format(IDS_ERROR_INVALID_PARAMETER,(LPCTSTR)ParamsArray[iIndex]+4);
						ErrorMessage(msg);
						return PROCESS_INVALID;
					}
				}else if(_T("/cp")==Parameter){
					uCodePage=UTILCP_SJIS;	//ftHgɖ߂
				}else{
					CString msg;
					msg.Format(IDS_ERROR_INVALID_PARAMETER,ParamsArray[iIndex]);
					ErrorMessage(msg);
					return PROCESS_INVALID;
				}
			}else if(0==_tcsncmp(_T("/c"),Parameter,2)){//kwĂ
				ProcessMode=PROCESS_COMPRESS;
				//----------------
				// k`̉
				//----------------
				if(_T("/c")==Parameter){	//`w肳ĂȂꍇ
					cli.CompressType=PARAMETER_UNDEFINED;
				}else if(0!=_tcsncmp(_T("/c:"),Parameter,3)){
					CString msg;
					msg.Format(IDS_ERROR_INVALID_PARAMETER,ParamsArray[iIndex]);
					ErrorMessage(msg);
					return PROCESS_INVALID;
				}else if(DLL_ID_B2E==cli.idForceDLL){	//B2Eňkꍇ
					//k̂ݐ؂肾
					cli.strFormat=(LPCTSTR)Parameter+3;
				}else{
					cli.CompressType=PARAMETER_UNDEFINED;
					//R}hCp[^ƌ`̑Ή\T
					for(int i=0;i<COMPRESS_PARAM_COUNT;i++){
						if(CompressParameterArray[i].Param==Parameter){
							cli.CompressType=CompressParameterArray[i].Type;
							cli.Options=CompressParameterArray[i].Options;
							break;
						}
					}
					if(PARAMETER_UNDEFINED==cli.CompressType){
						CString msg;
						msg.Format(IDS_ERROR_INVALID_COMPRESS_PARAMETER,Parameter);
						ErrorMessage(msg);
						return PROCESS_INVALID;
					}

					//CONTROLL[ĂȂAʈk
					if(bPressedControl){
						cli.bSingleCompression=true;
					}
				}
			}else if(_T("/e")==Parameter){//𓀂wĂ
				if(bPressedShift){
					ProcessMode=PROCESS_LIST;	//SHIFTL[Ă{[h
				}else if(bPressedControl){
					ProcessMode=PROCESS_TEST;	//CTRLL[Ă猟[h
				}else{
					ProcessMode=PROCESS_EXTRACT;
				}
			}else if(_T("/l")==Parameter){//t@Cꗗ\
				ProcessMode=PROCESS_LIST;
			}else if(_T("/t")==Parameter){//A[JCueXg
				ProcessMode=PROCESS_TEST;
			}else if(_T("/m")==Parameter){//@I
				CConfigOpenAction ConfOpenAction;
				ConfOpenAction.load(ConfigManager);
				OPENACTION OpenAction;
				if(bPressedShift){	//---Shift
					OpenAction=ConfOpenAction.OpenAction_Shift;
				}else if(bPressedControl){	//---Ctrl
					OpenAction=ConfOpenAction.OpenAction_Ctrl;
				}else{	//---ʏ펞
					OpenAction=ConfOpenAction.OpenAction;
				}
				switch(OpenAction){
				case OPENACTION_EXTRACT://
					ProcessMode=PROCESS_EXTRACT;
					break;
				case OPENACTION_LIST:	//{
					ProcessMode=PROCESS_LIST;
					break;
				case OPENACTION_TEST:	//
					ProcessMode=PROCESS_TEST;
					break;
				case OPENACTION_ASK:	//mF
					ProcessMode=SelectOpenAction();
					if(ProcessMode==PROCESS_INVALID){
						return PROCESS_INVALID;
					}
					break;
				default:
					ASSERT(!"This code must not be run");
					return PROCESS_INVALID;
				}
			}else if(_T("/!")==Parameter||_T("/xacrett")==Parameter){//XacRett.DLLgp
				cli.idForceDLL=DLL_ID_XACRETT;
			}else if(_T("/b2e")==Parameter){//B2E32.dllgp
				cli.idForceDLL=DLL_ID_B2E;
			}else if(_T("/b2esfx")==Parameter){//B2E32.dllgpĎȉ𓀂Ɉk
				cli.Options|=COMPRESS_SFX;
			}else if(_T("/s")==Parameter){//t@Ck
				cli.bSingleCompression=true;
			}else if(0==_tcsncmp(_T("/o"),Parameter,2)){//o͐tH_w
				if(0==_tcsncmp(_T("/o:"),Parameter,3)){
					//o͐tH_̐؂o;̎_""͊OĂ
					cli.OutputDir=(LPCTSTR)Parameter+3;
					cli.OutputToOverride=(OUTPUT_TO)-1;
				}else if(_T("/o")==Parameter){
					cli.OutputDir.Empty();
					cli.OutputToOverride=(OUTPUT_TO)-1;
				}else if(_T("/od")==Parameter){
					//fXNgbvɏo
					cli.OutputDir.Empty();
					cli.OutputToOverride=OUTPUT_TO_DESKTOP;
				}else if(_T("/os")==Parameter){
					//fBNgɏo
					cli.OutputDir.Empty();
					cli.OutputToOverride=OUTPUT_TO_SAME_DIR;
				}else if(_T("/oa")==Parameter){
					//񕷂
					cli.OutputDir.Empty();
					cli.OutputToOverride=OUTPUT_TO_ALWAYS_ASK_WHERE;
				}else{
					CString msg;
					msg.Format(IDS_ERROR_INVALID_PARAMETER,ParamsArray[iIndex]);
					ErrorMessage(msg);
					return PROCESS_INVALID;
				}
				TRACE(_T("OutputDir=%s\n"),cli.OutputDir);
			}else if(0==_tcsncmp(_T("/@"),Parameter,2)&&Parameter.GetLength()>2){//X|Xt@Cw
				CString strFile;
				if(PATHERROR_NONE!=UtilGetCompletePathName(strFile,(LPCTSTR)Parameter+2)||
					!UtilReadFromResponceFile(strFile,uCodePage,cli.FileList)){
					//ǂݍݎs
					ErrorMessage(CString(MAKEINTRESOURCE(IDS_ERROR_READ_RESPONCEFILE)));
					return PROCESS_INVALID;
				}
			}else if(0==_tcsncmp(_T("/$"),Parameter,2)&&Parameter.GetLength()>2){//X|Xt@Cw(ǂݎ폜)
				CString strFile;
				if(PATHERROR_NONE!=UtilGetCompletePathName(strFile,(LPCTSTR)Parameter+2)||
					!UtilReadFromResponceFile(strFile,uCodePage,cli.FileList)){
					//ǂݍݎs
					ErrorMessage(CString(MAKEINTRESOURCE(IDS_ERROR_READ_RESPONCEFILE)));
					return PROCESS_INVALID;
				}
				else DeleteFile(strFile);	//폜
			}else if(0==_tcsncmp(_T("/f"),Parameter,2)){//o̓t@Cw
				if(0==_tcsncmp(_T("/f:"),Parameter,3)){
					//o̓t@C̐؂o;̎_""͊OĂ
					cli.OutputFileName=(LPCTSTR)Parameter+3;
				}else if(_T("/f")==Parameter){
					cli.OutputFileName.Empty();
				}else{
					CString msg;
					msg.Format(IDS_ERROR_INVALID_PARAMETER,ParamsArray[iIndex]);
					ErrorMessage(msg);
					return PROCESS_INVALID;
				}
				TRACE(_T("OutputFileName=%s\n"),cli.OutputFileName);
			}else if(0==_tcsncmp(_T("/method:"),Parameter,8)){//k\bhw
				cli.strMethod=(LPCTSTR)Parameter+8;
			}else if(0==_tcsncmp(_T("/level:"),Parameter,7)){//k\bhw
				cli.strLevel=(LPCTSTR)Parameter+7;
			}else if(0==_tcsncmp(_T("/mkdir:"),Parameter,7)){//𓀎̏o̓fBNg
				CString mode=(LPCTSTR)Parameter+7;
				if(_T("no")==mode){
					cli.CreateDirOverride=CREATE_OUTPUT_DIR_NEVER;
				}else if(_T("single")==mode){
					cli.CreateDirOverride=CREATE_OUTPUT_DIR_SINGLE;
				}else if(_T("always")==mode){
					cli.CreateDirOverride=CREATE_OUTPUT_DIR_ALWAYS;
				}else{
					CString msg;
					msg.Format(IDS_ERROR_INVALID_PARAMETER,ParamsArray[iIndex]);
					ErrorMessage(msg);
					return PROCESS_INVALID;
				}
			}else if(0==_tcsncmp(_T("/popdir:"),Parameter,8)){//𓀎̏o̓fBNg
				CString mode=(LPCTSTR)Parameter+8;
				if(_T("no")==mode){
					cli.IgnoreTopDirOverride=0;
				}else if(_T("yes")==mode){
					cli.IgnoreTopDirOverride=1;
				}else{
					CString msg;
					msg.Format(IDS_ERROR_INVALID_PARAMETER,ParamsArray[iIndex]);
					ErrorMessage(msg);
					return PROCESS_INVALID;
				}
			}else if(_T("/popdir")==Parameter){//𓀎̏o̓fBNg
				cli.IgnoreTopDirOverride=1;
			}else if(0==_tcsncmp(_T("/delete:"),Parameter,8)){//Ƀ\[X폜邩
				CString mode=(LPCTSTR)Parameter+8;
				if(_T("no")==mode){
					cli.DeleteAfterProcess=0;
				}else if(_T("yes")==mode){
					cli.DeleteAfterProcess=1;
				}else{
					CString msg;
					msg.Format(IDS_ERROR_INVALID_PARAMETER,ParamsArray[iIndex]);
					ErrorMessage(msg);
					return PROCESS_INVALID;
				}
			}else if(_T("/delete")==Parameter){//ɍ폜
				cli.DeleteAfterProcess=1;
			}else if(0==_tcsncmp(_T("/priority:"),Parameter,10)){	//vZXDx
				CString mode=(LPCTSTR)Parameter+10;
					 if(_T("low")==mode)	cli.PriorityOverride=LFPRIOTITY_LOW;
				else if(_T("lower")==mode)	cli.PriorityOverride=LFPRIOTITY_LOWER;
				else if(_T("normal")==mode)	cli.PriorityOverride=LFPRIOTITY_NORMAL;
				else if(_T("higher")==mode)	cli.PriorityOverride=LFPRIOTITY_HIGHER;
				else if(_T("high")==mode)	cli.PriorityOverride=LFPRIOTITY_HIGH;
				else{
					CString msg;
					msg.Format(IDS_ERROR_INVALID_PARAMETER,ParamsArray[iIndex]);
					ErrorMessage(msg);
					return PROCESS_INVALID;
				}
			}else{	//m̃IvV
				CString msg;
				msg.Format(IDS_ERROR_INVALID_PARAMETER,ParamsArray[iIndex]);
				ErrorMessage(msg);
				return PROCESS_INVALID;
			}
		}
	}
	if(cli.FileList.empty()){
		//XCb`݂̂w肳Ăꍇɂ͐ݒʂ\
		//------
		// ݂Ȃt@Cw肳Ăꍇɂ̓G[ԂĂ̂ŁA
		// Ńt@CXgł΃t@Cw肳ĂȂƔfłB
		//return PROCESS_CONFIGURE;
	}else{
		//ChJ[h̓WJ
		UtilPathExpandWild(cli.FileList,cli.FileList);
	}
	//---t@C̃tpXȂǃ`FbN
	for(std::list<CString>::iterator ite=cli.FileList.begin();ite!=cli.FileList.end();ite++){
		CPath strAbsPath;
		switch(UtilGetCompletePathName(strAbsPath,*ite)){
		case PATHERROR_NONE:
			//
			break;
		case PATHERROR_INVALID:
			//p[^w肪s
			ASSERT(!"p[^w肪s");
			return PROCESS_INVALID;
		case PATHERROR_ABSPATH:
			//΃pX̎擾Ɏs
			ErrorMessage(CString(MAKEINTRESOURCE(IDS_ERROR_FAIL_GET_ABSPATH)));
			return PROCESS_INVALID;
		case PATHERROR_NOTFOUND:
			//t@C̓tH_݂Ȃ
			{
				CString msg;
				msg.Format(IDS_ERROR_FILE_NOT_FOUND,*ite);
				ErrorMessage(msg);
			}
			return PROCESS_INVALID;
		case PATHERROR_LONGNAME:
			//Ot@C擾s
			ErrorMessage(CString(MAKEINTRESOURCE(IDS_ERROR_FAIL_GET_LONGNAME)));
			return PROCESS_INVALID;
		}

		//pX̍Ōオ\ŏIĂ\菜
		strAbsPath.RemoveBackslash();
		TRACE(strAbsPath),TRACE(_T("\n"));

		//lXV
		*ite=(CString)strAbsPath;
	}
	//o̓tH_w肳ĂA΃pXɕϊ
	if(!cli.OutputDir.IsEmpty()){
		if(!UtilGetAbsPathName(cli.OutputDir,cli.OutputDir)){
			//΃pX̎擾Ɏs
			ErrorMessage(CString(MAKEINTRESOURCE(IDS_ERROR_FAIL_GET_ABSPATH)));
			return PROCESS_INVALID;
		}
	}

	return ProcessMode;
}

