/*
 * Copyright (c) 2005-2012, Claybird
 * All rights reserved.

 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:

 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the Claybird nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#include "stdafx.h"
#include "../ArchiverCode/arc_interface.h"
#include "ConfigManager.h"
#include "ConfigExtract.h"
#include "../Utilities/FileOperation.h"

void CConfigExtract::load(CONFIG_SECTION &Config)
{
	OutputDirType=(OUTPUT_TO)Config.Data[_T("OutputDirType")].GetNParam(0,OUTPUT_TO_LAST_ITEM,OUTPUT_TO_DESKTOP);

	CString Buffer=Config.Data[_T("OutputDir")];
	if(!Buffer.IsEmpty()){
		UtilGetCompletePathName(OutputDir,Buffer);
	}else{
		OutputDir=_T("");
	}

	//𓀌tH_Jǂ
	OpenDir=Config.Data[_T("OpenFolder")].GetNParam(TRUE);

	//𓀎tH_dɂ邩ǂ
	CreateDir=(CREATE_OUTPUT_DIR)Config.Data[_T("CreateDir")].GetNParam(0,CREATE_OUTPUT_DIR_LAST_ITEM,CREATE_OUTPUT_DIR_ALWAYS);

	//𓀎t@CmFɏ㏑邩ǂ
	ForceOverwrite=Config.Data[_T("ForceOverwrite")].GetNParam(FALSE);

	//𓀎tH_琔ƋL폜
	RemoveSymbolAndNumber=Config.Data[_T("RemoveSymbolAndNumber")].GetNParam(FALSE);

	//𓀎t@CEtH_̎tH_Ȃ
	CreateNoFolderIfSingleFileOnly=Config.Data[_T("CreateNoFolderIfSingleFileOnly")].GetNParam(FALSE);

	//ɉ𓀂t@C𐧌
	LimitExtractFileCount=Config.Data[_T("LimitExtractFileCount")].GetNParam(FALSE);

	//ɉ𓀂t@C̏
	MaxExtractFileCount=max(1,Config.Data[_T("MaxExtractFileCount")].GetNParam(1));

	//ɉ𓀂łkt@C폜
	DeleteArchiveAfterExtract=Config.Data[_T("DeleteArchiveAfterExtract")].GetNParam(FALSE);

	//𓀌t@CݔɈړ
	MoveToRecycleBin=Config.Data[_T("MoveToRecycleBin")].GetNParam(TRUE);

	//mFɃA[JCu폜/ݔɈړ
	DeleteNoConfirm=Config.Data[_T("DeleteNoConfirm")].GetNParam(FALSE);

	//𓀃G[młȂꍇ폜
	ForceDelete=Config.Data[_T("ForceDelete")].GetNParam(FALSE);

	//}`{[폜
	DeleteMultiVolume=Config.Data[_T("DeleteMultiVolume")].GetNParam(FALSE);

	//pX[h͉񐔂ŏɂȂTRUE
	MinimumPasswordRequest=Config.Data[_T("MinimumPasswordRequest")].GetNParam(FALSE);

	//𓀑ΏۂOgq
	if(has_key(Config.Data,_T("DenyExt"))){
		DenyExt=Config.Data[_T("DenyExt")];
	}else{
		DenyExt.LoadString(IDS_DENYEXT_DEFAULT);
	}
}

void CConfigExtract::store(CONFIG_SECTION &Config)const
{
	Config.Data[_T("OutputDirType")]=OutputDirType;

	Config.Data[_T("OutputDir")]=OutputDir;

	//𓀌tH_Jǂ
	Config.Data[_T("OpenFolder")]=OpenDir;

	//𓀎tH_dɂ邩ǂ
	Config.Data[_T("CreateDir")]=CreateDir;

	//𓀎t@CmFɏ㏑邩ǂ
	Config.Data[_T("ForceOverwrite")]=ForceOverwrite;

	//𓀎tH_琔ƋL폜
	Config.Data[_T("RemoveSymbolAndNumber")]=RemoveSymbolAndNumber;

	//𓀎t@CEtH_̎tH_Ȃ
	Config.Data[_T("CreateNoFolderIfSingleFileOnly")]=CreateNoFolderIfSingleFileOnly;

	//ɉ𓀂t@C𐧌
	Config.Data[_T("LimitExtractFileCount")]=LimitExtractFileCount;

	//ɉ𓀂t@C̏
	Config.Data[_T("MaxExtractFileCount")]=MaxExtractFileCount;

	//ɉ𓀂łkt@C폜
	Config.Data[_T("DeleteArchiveAfterExtract")]=DeleteArchiveAfterExtract;

	//𓀌t@CݔɈړ
	Config.Data[_T("MoveToRecycleBin")]=MoveToRecycleBin;

	//mFɃA[JCu폜/ݔɈړ
	Config.Data[_T("DeleteNoConfirm")]=DeleteNoConfirm;

	//𓀃G[młȂꍇ폜
	Config.Data[_T("ForceDelete")]=ForceDelete;

	//}`{[폜
	Config.Data[_T("DeleteMultiVolume")]=DeleteMultiVolume;

	//pX[h͉񐔂ŏɂȂTRUE
	Config.Data[_T("MinimumPasswordRequest")]=MinimumPasswordRequest;

	//𓀑ΏۂOgq
	Config.Data[_T("DenyExt")]=DenyExt;
}

void CConfigExtract::load(CConfigManager &ConfMan)
{
	load(ConfMan.GetSection(_T("Extract")));
}

void CConfigExtract::store(CConfigManager &ConfMan)const
{
	store(ConfMan.GetSection(_T("Extract")));
}
